class LearningObjectBuilder < Builder
  ##
  # receive a list of ids and return a list of learning objects
  # useful for search pagination
  #
  def self.build(objects = [])
    super(learning_object_repository, objects, true, 'rid')
  end

  def self.build_from_orientdb(args = {})
    super(args) do
      # args['metadata'] = []
      args['metadata'] = JSON.parse(args['metadata']) if args['metadata'].class == String
      obj = Orientdb::LearningObject.new(
          id: args['@rid'],
          name: args['name'],
          description: args['description'],
          thumbnail: (args['thumbnail'] or ''),
          id_dspace: args['id_dspace'],
          type: args['type'],
          attachments: build_bitstreams_collection(args['bitstreams']),
          last_modified: args['last_modified'],
          metadata: args['metadata']
      )
      obj.created_at = DateTime.strptime(args['created_at'], "%Y-%m-%d %H:%M:%S") unless args['created_at'].blank?
      obj.last_modified = DateTime.strptime(args['last_modified'], "%Y-%m-%d %H:%M:%S") unless args['last_modified'].blank?
      obj.published_at = DateTime.strptime(args['published_at'], "%Y-%m-%d %H:%M:%S") unless args['published_at'].blank?
      obj.publisher = learning_object_repository.get_publisher(obj)
      obj.subjects = learning_object_repository.get_subjects(obj)
      obj.attributes = learning_object_repository.get_attributes(obj)
      obj
    end
  end

  private

  def self.build_bitstreams_collection(bitstreams)
    bitstreams.map {|bitstream| ::LearningObject::AttachmentBuilder.build_from_orientdb(bitstream) }
  end

end
