class View < ActiveRecord::Base
  belongs_to :viewable, polymorphic: true, counter_cache: true
  belongs_to :user

  scope :created_last, ->(user) { where(user: user).order('created_at DESC').limit(1).first }

  before_create :current_time_greater_than_last

  private

  def current_time_greater_than_last
    last_view = viewable.views.created_last(user)

    unless last_view.blank?
      return false if Time.now < (last_view.created_at + 1.days)
    end

    true
  end
end
