class OrientDb::Base
  include OrientDb::Methods::FinderMethods
  include OrientDb::Methods::CrudMethods
  include OrientDb::Methods::SanitizableMethods

  def initialize(orientdb_connection)
    @connection = orientdb_connection
  end

  def connection
    @connection
  end

  protected

  def build_objects(hash=[])
    objects = []
    hash.each do |h|
      objects << build_object(h)
    end
    objects
  end

  def build_object(args={})
    builder = "#{args['@class']}Builder".constantize
    builder.build_from_orientdb args unless args['@class'].blank? || builder.nil?
  end

  def accepted_properties
    raise NoMethodError, "You must implement this method"
  end

  def odb_class
    fullname = self.class.to_s.split('Repository').first
    fullname.split('::').last.to_s
  end

end
