class SearchService
  include Log::Logging
  include RepositoriesProxy

  def initialize(orientdb_connection)
    @connection = orientdb_connection
    
   #logger.level = Log::DatabaseLogger::SELECT
  end

  def search(params)
    cache_key = build_search_cache_key params

    #logger.info("\n\n params\n" + params.to_s) 
    #logger.info("\n\n cache_key\n" + cache_key.to_s);

    # get results in cache or search
    Rails.cache.fetch(cache_key, expires_in: 10.minutes) do
      query = build_search_query params
      # p query
      @connection.query query, limit: 10000
    end
  end

  private

  def build_search_query(params={})
    order = order_by(params[:order])
    ord_attr = order.split(' ', 2).first
    qry = "SELECT @rid.asString(), last_modified, @class, #{ord_attr} FROM ("+main_query(params)+") ORDER BY #{order}"
    
    #logger.info("\n\n QUERY\n#{qry}")
    qry
  end

  def main_query(params={})
    "SELECT EXPAND($c) LET $a = ("+collection_query(params)+"), $b = ("+learning_object_query(params)+"), $c = UNIONALL($a, $b)"
  end

  def collection_query(params={})
    qry = "SELECT *, created_at as published_at, out('BelongsTo')[0].name as author FROM Collection"
    qry += " WHERE name LUCENE '#{params[:query]}' OR description LUCENE '#{params[:query]}'" unless params[:query].blank?
    qry = "SELECT FROM (#{qry}) WHERE privacy = 'public'"
    filter = collection_learning_objects_filter(params)
    qry += " AND "+filter unless filter.blank?
    qry
  end

  def learning_object_query(params={})
    qry = "SELECT FROM LearningObject"

    qry +=" WHERE name LUCENE '#{params[:query]}' OR description LUCENE '#{params[:query]}' OR author LUCENE '#{params[:query]}'" unless params[:query].blank?
    filter = learning_object_filters(params)
    qry = "SELECT FROM (#{qry}) WHERE "+filter unless filter.blank?
    qry
  end

  def collection_learning_objects_filter(params={})
    qry = learning_object_filters(params)
    qry = "learning_objects CONTAINS (#{qry})" unless qry.blank?
    qry
  end

  def learning_object_filters(params={})
    qry = ""
    qry = " AND ("+subject_filter(params[:subject])+")" unless params[:subject].blank?
    unless params[:school_level].blank?
      params[:school_level] << 'Ensino Fundamental Inicial' << 'Ensino Fundamental Final' if params[:school_level].include? 'Ensino Fundamental'
      qry += " AND ("+subject_filter(params[:school_level])+")"
    end
    qry += " AND ("+type_filter(params[:type])+")" unless params[:type].blank?
    qry += " AND ("+year_filter(params[:year])+")" unless params[:year].blank?
    qry += " AND ("+source_filter(params[:source])+")" unless params[:source].blank?

    qry[5..-1]
  end

  def subject_filter(subjects)
    "out('IsAbout') CONTAINS (name in ['"+subjects.join("','")+"'])"
  end

  def source_filter(sources)
    # SELECT FROM LearningObject WHERE out('PublishedBy') CONTAINS (@rid='#16:1')

    qry = "out('PublishedBy') CONTAINS (name in ['"+sources.split("-s-").join("','")+"'])"
    qry
  end

  def year_filter(years)
    if years.size() == 1
      year = years[0]
      qry = "created_at.format('yyyy') = '#{year}'"
    elsif years.size() == 2
      begin_year = years[0]
      end_year = years[1]
      qry = "created_at.format('yyyy') >= '#{begin_year}' AND created_at.format('yyyy') <= '#{end_year}'"
    end
  end

  def type_filter(types)
    qry = ''
    types.each do |type|
      qry = qry + "type = '#{type}' " if type == types.first
      qry = qry + "OR type = '#{type}' " if type != types.first
    end
    qry
  end

  def order_by(order)
    case order
      when 'author'
        'author ASC'
      when 'publicationasc'
        'published_at ASC'
      when 'publicationdesc'
        'published_at DESC'
      when 'title'
        'name ASC'
      else
        'score DESC'
    end
  end

  def accepted_properties
    ['thumbnail', 'score']
  end

  def search_params
    %w(query order subject type school_level year source)
  end

  def build_search_cache_key(params={})
    cache_key = "search_result/"
    search_params.each do |param|
      if params[param].class == Array
        cache_key += params[param].join('-')
      else
        cache_key += params[param].to_s
      end
      cache_key += '/' unless (param == search_params.last)
    end
    cache_key
  end

end